﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2010 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.RealVectorEncoding;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Algorithms.ParticleSwarmOptimization {

  [StorableClass]
  public class ParticleUpdater : SingleSuccessorOperator {

    #region Parameter properties
    public LookupParameter<IRandom> RandomParameter {
      get { return (LookupParameter<IRandom>)Parameters["Random"]; }
    }
    public LookupParameter<RealVector> PointParameter {
      get { return (LookupParameter<RealVector>)Parameters["Point"]; }
    }
    public LookupParameter<RealVector> VelocityParameter {
      get { return (LookupParameter<RealVector>)Parameters["Velocity"]; }
    }
    public LookupParameter<RealVector> PersonalBestPointParameter {
      get { return (LookupParameter<RealVector>)Parameters["PersonalBestPoint"]; }
    }
    public LookupParameter<RealVector> BestPointParameter {
      get { return (LookupParameter<RealVector>)Parameters["BestPoint"]; }
    }
    public LookupParameter<DoubleMatrix> BoundsParameter {
      get { return (LookupParameter<DoubleMatrix>)Parameters["Bounds"]; }
    }
    public LookupParameter<DoubleMatrix> VelocityBoundsParameter {
      get { return (LookupParameter<DoubleMatrix>)Parameters["VelocityBounds"]; }
    }
    public LookupParameter<DoubleValue> OmegaParameter {
      get { return (LookupParameter<DoubleValue>)Parameters["Omega"]; }
    }
    public LookupParameter<DoubleValue> Phi_PParameter {
      get { return (LookupParameter<DoubleValue>)Parameters["Phi_P"]; }
    }
    public LookupParameter<DoubleValue> Phi_GParameter {
      get { return (LookupParameter<DoubleValue>)Parameters["Phi_G"]; }
    }
    #endregion

    #region Parameter Values
    public IRandom Random {
      get { return RandomParameter.ActualValue; }
    }
    public RealVector Point {
      get { return PointParameter.ActualValue; }
      set { PointParameter.ActualValue = value; }
    }
    public RealVector Velocity {
      get { return VelocityParameter.ActualValue; }
      set { VelocityParameter.ActualValue = value; }
    }
    public RealVector PersonalBestPoint {
      get { return PersonalBestPointParameter.ActualValue; }
    }
    public RealVector BestPoint {
      get { return BestPointParameter.ActualValue; }
    }
    public DoubleMatrix Bounds {
      get { return BoundsParameter.ActualValue; }
    }
    public DoubleMatrix VelocityBounds {
      get { return VelocityBoundsParameter.ActualValue; }
    }
    public double Omega {
      get { return OmegaParameter.ActualValue.Value; }
    }
    public double Phi_P {
      get { return Phi_PParameter.ActualValue.Value; }
    }
    public double Phi_G {
      get { return Phi_GParameter.ActualValue.Value; }
    }
    #endregion

    #region Construction & Cloning

    [StorableConstructor]
    protected ParticleUpdater(bool deserializing) : base(deserializing) { }
    protected ParticleUpdater(ParticleUpdater original, Cloner cloner)
      : base(original, cloner) {
    }

    public ParticleUpdater()
      : base() {
      Parameters.Add(new LookupParameter<IRandom>("Random", "Random number generator."));
      Parameters.Add(new LookupParameter<RealVector>("Point", "Particle's current position"));
      Parameters.Add(new LookupParameter<RealVector>("Velocity", "Particle's current velocity."));
      Parameters.Add(new LookupParameter<RealVector>("PersonalBestPoint", "Particle's personal best position"));
      Parameters.Add(new LookupParameter<RealVector>("BestPoint", "Global best position"));
      Parameters.Add(new LookupParameter<DoubleMatrix>("Bounds", "The lower and upper bounds for each dimension of the position vector for the current problem."));
      Parameters.Add(new LookupParameter<DoubleMatrix>("VelocityBounds", "Upper and lower bounds for the particle's velocity vector."));
      Parameters.Add(new LookupParameter<DoubleValue>("Omega", "The weight for the particle's velocity vector."));
      Parameters.Add(new LookupParameter<DoubleValue>("Phi_P", "The weight for the particle's personal best position."));
      Parameters.Add(new LookupParameter<DoubleValue>("Phi_G", "The weight for the global best position."));
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new ParticleUpdater(this, cloner);
    }

    #endregion

    public override IOperation Apply() {
      RealVector velocity = new RealVector(Velocity.Length);
      RealVector position = new RealVector(Point.Length);
      double r_p = Random.NextDouble();
      double r_g = Random.NextDouble();
      for (int i = 0; i < velocity.Length; i++) {
        velocity[i] =
          Velocity[i] * Omega +
          (PersonalBestPoint[i] - Point[i]) * Phi_P * r_p +
          (BestPoint[i] - Point[i]) * Phi_G * r_g;
      }
      BoundsChecker.Apply(velocity, VelocityBounds);
      for (int i = 0; i < velocity.Length; i++) {
        position[i] = Point[i] + velocity[i];
      }
      BoundsChecker.Apply(position, Bounds);
      Point = position;
      Velocity = velocity;

      return base.Apply();
    }

    public override bool CanChangeName {
      get { return false; }
    }
  }
}
